/* @(#) pSOSystem ARM/V2.2.1: bin/source/aif2hex.c  1.2 97/05/14 15:51:49 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/aif2hex.c                                     */
/*   DATE:    97/05/14                                                 */
/*   PURPOSE: Utility program to convert AIF to HEX file               */
/*            SOURCE: ARM, Ed Nevill                                   */
/*            Modified by GChevalier ISI                               */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1996, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
/*
 * RCS $Revision: 1.2 $
 * Checkin $Date: 1995/03/29 15:52:13 $
 * Revising $Author: enevill $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifndef __BORLANDC__
#define __STDC__
#endif

#define help_line1      "Command format: AIF2Hex [<options>] -<format> <binary-file> [<hex-output>]\n"
#define help_line2      "\nOptions:\n"
#define help_line3      "-Base n       Base address in output file\n              (prefix 'n' with 0x for hex, follow with K for kilobytes)\n"
#define help_line4	"-Bin          Accept a plain binary file as input rather than an AIF file\n"
#define help_line5      "\nFormat may be one of the following:\n"
#define help_line6      "-Cosmac       RCA Cosmac format\n"
#define help_line7      "-Fairbug      Fairchild Fairbug format\n"
#define help_line8      "-MOSTech      MOS Technology format\n"
#define help_line9      "-EXORciser    Motorola EXORciser format (16 bit 'S' records)\n"
#define help_line10     "-MDS          Intel Intellec 8/MDS format\n"
#define help_line11     "-IHF          Extended Intellec Hex format\n"
#define help_line12     "-Signetics    Signetics Absolute Object format\n"
#define help_line13     "-Tektronix    Tektronix Hexadecimal format\n"
#define help_line14     "-EXORmacs     Motorola EXORmacs format (24 bit 'S' records)\n"
#define help_line15     "-MCS86        Intel MCS-86 Hexadecimal format\n"
#define help_line16     "-SDSMAC       Texas Instruments SDSMAC format\n"
#define help_line17     "-Motorola32   Motorola 32-Bit format (32 bit 'S' records)\n"
#define help_line18     "-Intel32      Intel Hex-32 format\n"

#define too_big         "Base and/or size too big for this format, max = 0x%X\n"
#define msg_usage       "usage: AIF2Hex -<format> <binary-file> [<output-file>]\n"
#define no_arg_on_base  "No argument on -base option\n"
#define bad_arg_on_base "Bad number on -base option\n"
#define mult_formats    "Duplicate formats, -%s & -%s\n"
#define bad_format      "Unrecognised format or option, -%s, enter AIF2Hex -help for help\n"
#define too_many_args   "Too many arguments\n"
#define no_format       "No format specified (use -help for help)\n"
#define err_openout     "Error opening %s for output\n"
#define err_openin      "Error opening %s for input\n"
#define err_seek        "Error seeking on %s\n"
#define err_ftell       "Error reading size of %s\n"
#define err_alloc       "Unable to allocate %d bytes\n"
#define err_read        "Error reading %s\n"
#define non_bin_aif     "warning: non binary AIF file, header not stripped\n"
#define non_aif_image   "warning: not an AIF image, plain binary assumed\n"
#define root_only_base  "warning: base will apply only to root segment\n"
#define root_only_name  "warning: output file name will only apply to root segment\n"

#ifndef SEEK_SET
#define SEEK_SET        0
#define SEEK_CUR        1
#define SEEK_END        2
#endif

#define COSMAC          1
#define FAIRBUG         2
#define MOSTECH         3
#define EXORCISER       4
#define MDS             5
#define SIGNETICS       6
#define TEKTRONIX       7
#define EXORMAX         8
#define MCS86           9
#define SDSMAC          10
#define MOTOROLA32      11
#define INTEL32         12
#define IHF             13

#define N_FORMATS       14

#define VERSION_STRING  "              <<< AIF2HEX Version 1.2 >>>\n\n"

#ifdef __STDC__
#else
static void print_trailer( int, int );
static int add_checksum(int, int, int);
static void print_trailer( int, int);
static void print_header( long, int, char *, unsigned long );
static void bin2hex( unsigned char *, long, int, char *, long);
static long getnum(char *arg, char *err);
static int cistrcmp(char *s, char *t);
static void give_help( void );
static void usage(char *s);
static unsigned long byte_x(unsigned long v);
#endif

typedef struct {
    unsigned long next_fragment_offset;
    unsigned long load_address;
    unsigned long size;
    char name[32];
} fragment_header;

char *formats[N_FORMATS] = {
    "",
    "cosmac",
    "fairbug",
    "mostech",
    "exorciser",
    "mds",
    "signetics",
    "tektronix",
    "exormacs",
    "mcs86",
    "sdsmac",
    "motorola32",
    "intel32",
    "ihf"
};

#ifdef __STDC__
static int add_checksum( checksum, format, v )
int checksum;
int format;
int v;	
{
#else
static int add_checksum(int checksum, int format, int v)
{
#endif
	
    int i, d;

    switch (format) {
      case COSMAC:      /* Cosmac has no checksum? */
        return 0;
      case TEKTRONIX:
      case FAIRBUG:
        return checksum + ((v >> 4) & 0x0f) + (v & 0x0f);
      case MOSTECH:
      case EXORCISER:
      case MDS:
      case EXORMAX:
      case MCS86:
      case MOTOROLA32:
      case INTEL32:
      case IHF:
        return checksum + v;
      case SIGNETICS:
        v = checksum ^ v;
        return (v << 1) | ((v >> 7) & 1);
      case SDSMAC:
        for (i = 0; i < 4; i++) {
            d = v & 0x0f;
            checksum += d >= 10 ? 'A' + d - 10 : '0' + d;
            v >>= 4;
        }
        return checksum;
    }
    return (-1);
}


#ifdef __STDC__
static void print_header( size, format, filename, base )
long size;
int format;
char *filename;
unsigned long base;
{
#else
static void print_header( long size, int format, char *filename,
                          unsigned long base)
{
#endif

    unsigned long limit;

    if (format == IHF)
        limit = 0x3ffff;
    else if (format == MCS86)
        limit = 0xfffff;
    else if (format == MOTOROLA32 || format == INTEL32)
        limit = 0xffffffff;
    else
        limit = 0xffff;
    if (base > limit || (base + size) > limit)
        {
        printf(too_big);
        exit(0);
        }
    switch (format) {
        case COSMAC:
            printf("!M%04X ,\n", (int)base);
            break;
        case FAIRBUG:
            printf("S%04X\n", (int)base);
            break;
        case MCS86: {
            int segment;

            segment = (base >> 4) & 0xf000;
            printf(":02000002%04X%02X\n", (int)segment, -(4+(segment & 0xff)+((segment>>8)& 0xff)) & 0xff);
            break;
        }
        case SDSMAC: {
            char buff[80];
            char *s, *t;
            int checksum;

            sprintf(buff, "0%04X7", (int)size);
            s = filename;
            t = buff+strlen(buff);
            while (*s) {
                sprintf(t, "%02X", toupper(*s++));
                t += 2;
            }
            checksum = '8';
            s = buff;
            while (*s) checksum += *s++;
            sprintf(t, "8%04XF", checksum & 0xffff);
            puts(buff);
            break;
        }
        case INTEL32: {
            int segment;

            segment = base >> 16;
            printf(":02000004%04X%02X\n", (int)(segment & 0xffff), -(6+(segment & 0xff)+((segment>>8)& 0xff)) & 0xff);
            break;
        }
    }
}
#ifdef __STDC__
static void print_trailer( n, format)
int n;
int format;	
{
#else
static void print_trailer( int n, int format)
{
#endif

    switch (format) {
      case COSMAC:
	printf("\n");
        break;
      case FAIRBUG:
        printf("*\n");
        break;
      case MOSTECH:
        printf(";00%04X%04X\n", n, n);
        break;
      case EXORMAX:
      case EXORCISER:
        printf("S9030000FC\n");
        break;
      case MCS86:
      case MDS:
        printf(":00000001FF\n");
        break;
      case IHF:
        printf(":00 0000 01 FF\n");
        break;
      case SIGNETICS:
        printf(":000000\n");
        break;
      case TEKTRONIX:
        printf("/00000000\n");
        break;
      case SDSMAC:
        printf(":\n");
        break;
      case MOTOROLA32:
        printf("S70500000000FA\n");
        break;
      case INTEL32:
        printf(":00000001FF\n");
        break;
    }
}

#ifdef __STDC__
static void bin2hex( buff, size, format, filename, file_base )
unsigned char *buff;
long size;
int format;
char *filename;
long file_base;
{
#else
static void bin2hex( unsigned char *buff, long size, int format,
							char *filename, long file_base )
{
#endif

    int new_addr;
    int bytes, checksum;
    long i;
    long base;
    int v;
    int records =0;

#ifdef DEBUG
    fprintf(stderr, "file_base: 0x%lx, filename :%s:, size: %5ld\n",
		     file_base, filename, size);
#endif

    print_header(size, format, filename, file_base);
    new_addr = 1;
    i = 0;
    while (i < size) {
        if (new_addr) {
            records++;
            v = 16;
            if (format == IHF) v = 32;
            bytes = ((i + v) & ~(v-1)) - i;
            if (bytes > size - i) bytes = size - i;
            base = file_base + i;
            checksum = 0;
            checksum = add_checksum(checksum, format, ((base >> 8) & 0xff));
            checksum = add_checksum(checksum, format, (base & 0xff));
            checksum = add_checksum(checksum, format, bytes);
            switch (format) {
                case COSMAC:
                    break;
                case FAIRBUG:
                    bytes = 8;
                    fputc('X', stdout);
                    checksum = 0;
                    break;
                case MOSTECH:
                    printf(";%02X%04X", bytes, (int)base);
                    break;
                case EXORCISER:
                    printf("S1%02X%04X", bytes+3, (int)base);
                    checksum += 3;
                    break;
                case MDS:
                    printf(":%02X%04X00", bytes, (int)base);
                    break;
                case IHF:
                    printf(":%02X %04X 00 ", bytes/4, (int)(base/4));
                    checksum = 0;
                    checksum = add_checksum(checksum, format, bytes/4);
                    checksum = add_checksum(checksum, format, (((base/4) >> 8) & 0xff));
                    checksum = add_checksum(checksum, format, ((base/4) & 0xff));
                    break;
                case SIGNETICS:
                    printf(":%04X%02X%02X", (int)base, bytes, checksum & 0xff);
                    checksum = 0;
                    break;
                case TEKTRONIX:
                    printf("/%04X%02X%02X", (int)base, bytes, checksum & 0xff);
                    checksum = 0;
                    break;
                case EXORMAX:
                    printf("S2%02X%06lX", bytes+4, base);
                    checksum += 4;
                    checksum = add_checksum(checksum , format, (base >> 16) & 0xff);
                    break;
                case MCS86:
                    printf(":%02X%04X00", bytes, (int)(base & 0xffff));
                    break;
                case SDSMAC: {
                    char buff[16];
                    char *s;

                    sprintf(buff, "9%04X", (int)base);
                    checksum = 0;
                    s = buff;
                    fputs(s, stdout);
                    while (*s) checksum += *s++;
                    break;
                }
                case MOTOROLA32:
                    printf("S3%02X%08X", bytes + 5, base);
                    checksum += 5;
                    checksum = add_checksum(checksum, format, (base >> 16) & 0xff);
                    checksum = add_checksum(checksum, format, (base >> 24) & 0xff);
                    break;
                case INTEL32:
                    printf(":%02X%04X00", bytes, (int)(base & 0xffff));
                    break;
            }
            new_addr = 0;
        }
        v = i >= size ? 0xff : buff[i];
        i++;
        if (format == SDSMAC) {
            v = (v << 8) + (i >= size ? 0xff : buff[i]);
            i++;
            printf("B%04X", v);
            bytes--;
            checksum += 'B';
        } else
            printf("%02X", v);
        checksum = add_checksum(checksum, format, v);
        if (--bytes <= 0) {
            base = file_base + i;
            switch (format) {
                case COSMAC:
                    if (i < size) printf(",\n");
                    break;
                case FAIRBUG:
                    printf("%X\n", checksum & 0x0f);
                    break;
                case MOSTECH:
                    printf("%04X\n", checksum & 0xffff);
                    break;
                case EXORCISER:
                case EXORMAX:
                case MOTOROLA32:
                    printf("%02X\n", ~checksum & 0xff);
                    break;
                case IHF:
                    putchar(' ');
                case MDS:
                case MCS86:
                case INTEL32:
                    printf("%02X\n", -checksum & 0xff);
                    if ((base & 0xffff) == 0 && i < size)
                        print_header(0, format, 0, file_base + i);
                    break;
                case TEKTRONIX:
                case SIGNETICS:
                    printf("%02X\n", checksum &0xff);
                    break;
                case SDSMAC:
                    checksum += '8';
                    printf("8%04XF\n", -checksum & 0xffff);
                    break;
            }
            new_addr = 1;
        }
    }
    print_trailer(records, format);
}

#ifdef __STDC__
static int cistrcmp( s, t )
char *s;
char *t;
{
#else
static int cistrcmp(char *s, char *t)
{
#endif

    int sch, tch;

    for (;;) {
        sch = *s++;
        tch = *t++;
        if (sch == tch) {
            if (sch == 0) return 0; else continue;
        }
        sch = tolower(sch);
        tch = tolower(tch);
        if (sch != tch)  return sch - tch;
    }
}

#ifdef __STDC__
static long getnum( arg, err)
char *arg;
char *err;
{
#else
static long getnum(char *arg, char *err)
{
#endif

    long numbase = 10, n = 0;
    int c;

    if (arg[0] == '&' || arg[0] == '0' && (arg[1] == 'x' || arg[1] == 'X')) {
        numbase = 16;
        if (arg[0] == '&') arg += 1; else arg += 2;
    }
    while ((c = *arg++) != 0) {
        if (isdigit(c))
            n = n * numbase + c - '0';
        else {
            c = tolower(c);
            if (numbase != 16 || c < 'a' || c > 'f') break;
            n = n * numbase + c - 'a' + 10;
        }
    }
    if (c == 'k' || c == 'm') {
        n *= 1024;
        if (c == 'm') n *= 1024;
        c = *arg;
    }
    if (c)  { printf(err); exit(0);}

    return n;
}

#ifdef __STDC__
static unsigned long byte_x( v )
unsigned long v;
{
#else
static unsigned long byte_x(unsigned long v)
{
#endif
    
	 return (v << 24) | ((v << 8) & 0x00ff0000) | ((v >> 8) & 0x0000ff00) | (v >> 24);
}

#define byte_reverse(a) (byte_reversing ? byte_x(a) : (a))

#ifdef __STDC__
static void usage( s )
char *s;
{
#else
static void usage(char *s)
{
#endif

    fprintf(stderr, "%s", s);
    printf(msg_usage);
    exit(0);
}

#ifdef __STDC__
static int give_help()
{
#else
static void give_help( void )
{
#endif
	 fprintf(stderr, VERSION_STRING );
    fprintf(stderr, help_line1);
    fprintf(stderr, help_line2);
    fprintf(stderr, help_line3);
    fprintf(stderr, help_line4);
    fprintf(stderr, help_line5);
    fprintf(stderr, help_line6);
    fprintf(stderr, help_line7);
    fprintf(stderr, help_line8);
    fprintf(stderr, help_line9);
    fprintf(stderr, help_line10);
    fprintf(stderr, help_line11);
    fprintf(stderr, help_line12);
    fprintf(stderr, help_line13);
    fprintf(stderr, help_line14);
    fprintf(stderr, help_line15);
    fprintf(stderr, help_line16);
    fprintf(stderr, help_line17);
    fprintf(stderr, help_line18);
}

#define SWI_EXIT 0xef000011

#ifdef __STDC__
int main(  argc, argv )
int argc;
char **argv;
{
#else
int main( int argc, char **argv )
{
#endif

    char *infile = 0;
    char *outfile = 0;
    int format = 0;
    int size;
    FILE *f;
    char *s;
    unsigned char *buff;
    int i;
    long base = 0;                      /* gjc 3-20-97 */
    unsigned long w;
    int baseflag = 0;
    int binaryflag = 0;
    int byte_reversing;
    unsigned long *p;
    unsigned long fragment_offset;

    while ( (s = *++argv) != NULL )
    {
        if (*s == '-')
        {
            s++;
            if (cistrcmp(s, "help") == 0)
            {
                give_help();
                exit(0);
            }
            else if (cistrcmp(s, "base") == 0)
            {
                if ( (s = *++argv ) == NULL )
                {
                   printf(no_arg_on_base);
                   exit(0);
                }
                base = getnum(s, bad_arg_on_base);
                baseflag = 1;
            }
            else if (cistrcmp(s, "bin") == 0 || cistrcmp(s, "binary") == 0)
            {
                binaryflag = 1;
            }
            else
            {
                for (i = 1; i < N_FORMATS; i++)
                {
                    if (cistrcmp(s, formats[i]) == 0)
                    {
                        if (format)
                        {
                            printf(mult_formats, formats[format], formats[i]);
                            exit(0);
                        }
                        format = i;
                        break;
                    }
                }
                if (i == N_FORMATS)
                {
                    printf(bad_format, s);
                    exit(0);
                }
            }
        }
        else
        {
            if (!infile)
                infile = s;
            else if (!outfile)
                outfile = s;
            else
                usage(too_many_args);
        }
    }
    if (!format)
       usage(no_format);

    f = fopen(infile, "rb");
    if (!f)
    {
       printf(err_openin, infile);
       exit(0);
    }
    if (fseek(f, 0, SEEK_END))
    {
    	printf(err_seek, infile);
      exit(0);
    }
    size = ftell(f);
    if (size == -1)
    {
    	printf(err_ftell, infile);
      exit(0);
    }
    buff = (unsigned char *) malloc(size);
    if (!buff)
    {
       printf(err_alloc, size);
       exit(0);
    }
    rewind(f);
    if (fread(buff, 1, size, f) < size)
    {
       printf(err_read, infile);
       exit(0);
    }
    fclose(f);
    w = *(unsigned long *)(buff + 0x10);
    byte_reversing = 0;

    if (w == byte_x(SWI_EXIT))
    {
	     byte_reversing = 1;
        w = SWI_EXIT;
    }
    if (!binaryflag && w == SWI_EXIT)
    {
        unsigned long image_size;
        /* unsigned long debug_size;  */
        unsigned long image_entry;

        p = (unsigned long *)buff;
        image_size = byte_reverse(p[5] + p[6]);
	     image_entry = byte_reverse(p[3]);
       /* debug_size = byte_reverse(p[7]);  */
	     if (!baseflag)
	        base = byte_reverse(p[10]);
	        if ((image_entry & 0xff000000) != 0xeb000000)
              buff += 128;
           else
              fprintf(stderr, non_bin_aif);
	     fragment_offset = byte_reverse(p[14]);
	     if (fragment_offset)
        {
	        if (baseflag)
		        fprintf(stderr, root_only_base);
	        if (outfile)
		        fprintf(stderr, root_only_name);
	        else
		        outfile = "ROOT.hex";
	     }
        size = image_size;
    }
    else
    {
	    fragment_offset = 0;
	    if (!binaryflag)
	       fprintf(stderr, non_aif_image);
    }

    while (1)
    {
	    fragment_header *fragment;
	    char name[32+4];

	    if (outfile)
       {
	       if (!freopen(outfile, "w", stdout))
		    {
			   printf(err_openout, outfile); exit(0);
          }
	    }
       
	    bin2hex(buff, size, format, infile, base);

	    if (!fragment_offset)
          break;
	    fragment = (fragment_header *)((char *)p + fragment_offset);
	    fragment_offset = byte_reverse(fragment->next_fragment_offset);
	    base = byte_reverse(fragment->load_address);
	    size = byte_reverse(fragment->size);
	    strcpy(name, fragment->name);
	    strcat(name, ".hex");
	    outfile = name;
	    buff = (unsigned char *)fragment + sizeof(fragment_header);
    }
    return(0);
}
