/* @(#) pSOSystem ARM/V2.2.1: bin/source/cat.c  1.2 97/05/14 15:51:51 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/cat.c                                         */
/*   DATE:    97/05/14                                                 */
/*   PURPOSE: concatenate and list files                               */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1993, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*   The Makefiles used to build pSOSystem make use of the UNIX        */
/*   "cat" command.  This utility is supplied for use on host systems  */
/*   which do not have "cat", specifically MS-DOS and VMS.             */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <fcntl.h>

static void TurnSlashes(char *pathptr);

main(argc, argv)
int argc;
char *argv[];
{
char line[200], *ptr;
FILE *infile, *outfile;
int i;
int SkipArg1, SkipArg2;
char filemode[2];

/*---------------------------------------------------------------------*/
/* Convert UNIX-type filenames to DOS-type filenames                   */
/*---------------------------------------------------------------------*/
for (i = 1; i < argc; i++)
    TurnSlashes(argv[i]);

/*---------------------------------------------------------------------*/
/* Look for I/O re-direction, indicated by >filename.  If none found,  */
/* use standard out.                                                   */
/*---------------------------------------------------------------------*/
SkipArg1 = 0;
SkipArg2 = 0;
for (i = 1; i < argc; i++)
    {
    if (argv[i][0] == '>')
        {
        SkipArg1 = i;
        filemode[1] = 0;

        /*-------------------------------------------------------------*/
        /* Look to see if ">>" was given, which means append to the    */
        /* file.                                                       */
        /*-------------------------------------------------------------*/
        ptr = &(argv[i][1]);
        if (*ptr == '>')
            {
            filemode[0] = 'a';
            ptr++;
            }
        else
            filemode[0] = 'w';
        
        /*-------------------------------------------------------------*/
        /* If there was a space between the redirection symbol and     */
        /* the filename, they will be two separate arguments.          */
        /*-------------------------------------------------------------*/
        if (*ptr == 0)
            {
            ptr = argv[++i];
            SkipArg2 = i;
            }

        outfile = fopen(ptr, filemode);

        if (outfile == NULL)
            {
            fprintf(stderr, "Unable to open %s for output\n", argv[i][1]);
            exit(-1);
            }
        break;
        }
    }

if (SkipArg1 == 0)
    outfile = stdout;

/*---------------------------------------------------------------------*/
/* Now process each input file in turn.                                */
/*---------------------------------------------------------------------*/
for (i = 1; i < argc; i++)
    {
    if ((i == SkipArg1) || (i == SkipArg2))
        continue;

    infile = fopen(argv[i], "r");
    if (infile == NULL)
        {
        fprintf(stderr, "cat: Unable to open %s for input\n", argv[i]);
        continue;
        }

    while (fgets(line, 200, infile))
        fputs(line, outfile);

    fclose(infile);
    }

if (SkipArg1) fclose(outfile);
exit(0);
}

/***********************************************************************/
/* TurnSlashes: Convert a pathname to MS-DOS style                     */
/*                                                                     */
/*      INPUTS: pathptr: ptr to pathname to change                     */
/*                                                                     */
/***********************************************************************/
static void TurnSlashes(char *pathptr)
{
while (*pathptr)
    {
    if (*pathptr == '/') *pathptr = '\\';
    pathptr++;
    }
}
