/* @(#) pSOSystem ARM/V2.2.1: bin/source/ckdepend.c  1.2 97/05/14 15:51:52 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/ckdepend.c                                    */
/*   DATE:    97/05/14                                                 */
/*   PURPOSE: Utility to check whether a "target" file is newer than   */
/*            its dependency files, ala "make"                         */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1993, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*   This is a utility for checking an MS-DOS file against a list      */
/*   of "dependency" files to see if it should be rebuilt.             */
/*                                                                     */
/*   This utility can be used in conjunction with the build.bat        */
/*   files supplied with various projects to provide a "make"-like     */
/*   capability.   It                                                  */
/*                                                                     */
/*    ckdepend <target> <dep1> [<dep2>] [<dep3> ...]]                  */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <dos.h>
#include <direct.h>
#include <ctype.h>
#include <fcntl.h>

/*----------------------------------------------------------------------*/
/* Function prototypes                                                  */
/*----------------------------------------------------------------------*/
static void func_error(char *func, char *file);
static void TurnSlashes(char *pathptr);

/************************************************************************/
/*        main: Main body of the program                                */
/*                                                                      */
/************************************************************************/
main(argc, argv)
int argc;
char *argv[];
{
int handle;
unsigned int target_date, target_time;
unsigned int date, time;
int argnr;
struct find_t findbuff;
unsigned int rc;
char dir[60], filename[70];
char *ptr;

if ((*argv[1] == '?') || (argc < 3))
    {
    printf("CKDEPEND - file dependency checking utility V1.0  Copyright (c) 1992 - 1993, Integrated Systems, Inc.\n");
    printf("Syntax: ckdepend <target_file> <src_file1> [<src_file2> [...]]\n");
    exit(-1);
    }

/*---------------------------------------------------------------------*/
/* If we can't open the target file, then it MUST be rebuilt!          */
/*---------------------------------------------------------------------*/
TurnSlashes(argv[1]);
if (_dos_open(argv[1], O_RDONLY, &handle) != 0)
    exit(1);

/*---------------------------------------------------------------------*/
/* Get the date/time of the target file.                               */
/*---------------------------------------------------------------------*/
if (_dos_getftime(handle, &target_date, &target_time) != 0)
    func_error("_dos_getftime", argv[1]);

if (_dos_close(handle) != 0)
    func_error("_dos_close", argv[1]);

/*---------------------------------------------------------------------*/
/* Now check against each of the dependency files.  If one is found    */
/* that is newer than the target, the target must be rebuilt, so       */
/* return 1 to DOS.                                                    */
/*---------------------------------------------------------------------*/
for (argnr = 2; argnr < argc; argnr++)
    {
    TurnSlashes(argv[argnr]);

    /*-----------------------------------------------------------------*/
    /* Strip off the directory portion of this pathname.               */
    /*-----------------------------------------------------------------*/
    strcpy(dir, argv[argnr]);
    ptr = dir + strlen(dir) - 1;
    while (strlen(dir))
        if (*ptr != '\\')
            *ptr-- = 0;
        else
            break;
    
    rc = _dos_findfirst(argv[argnr], _A_NORMAL, &findbuff);
    while (rc == 0)
        {
        strcpy(filename, dir);
        strcat(filename, findbuff.name);
        if (_dos_open(filename, O_RDONLY, &handle) != 0)
            func_error("_dos_open", filename);

        if (_dos_getftime(handle, &date, &time) != 0)
            func_error("_dos_getftime", filename);

        if (_dos_close(handle) != 0)
            func_error("_dos_close", filename);
    
        if (date > target_date) 
            exit(1);
        else if ((date == target_date) && (time > target_time))
            exit(1);
        rc = _dos_findnext(&findbuff);
        }
    }
exit(0); 
}


/************************************************************************/
/*  func_error: Terminate due to an error on a file handling function   */
/*                                                                      */
/*      INPUTS: func: ptr to name of function that failed               */
/*              file: ptr to name of file failure occurred on           */
/*                                                                      */
/*        NOTE: Does not return!                                        */
/*                                                                      */
/************************************************************************/
static void func_error(func, file)
char *func;
char *file;
{
printf("CKDEPEND: %s error on %s\n", func, file);
exit(-1);
}


/***********************************************************************/
/* TurnSlashes: Convert a pathname to MS-DOS style                     */
/*                                                                     */
/*      INPUTS: pathptr: ptr to pathname to change                     */
/*                                                                     */
/***********************************************************************/
static void TurnSlashes(char *pathptr)
{
while (*pathptr)
    {
    if (*pathptr == '/') *pathptr = '\\';
    pathptr++;
    }
}
