/* @(#) pSOSystem ARM/V2.2.0: bin/source/make2bat/diskfile.c 2.3 95/12/13 19:32:41 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/make/diskfile.c                               */
/*   DATE:    95/12/13                                                 */
/*   PURPOSE: Routines for writing to and reading from the             */
/*            temporary disk file                                      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1995, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
#include "global.h"
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <stdio.h>
#include <string.h>

#define LINELEN 2000

static int TmpFile;

/***********************************************************************/
/*    FileInit: Initialize the temporary file                          */
/*                                                                     */
/***********************************************************************/
void FileInit(void)
{
TmpFile = open("make2bat.tmp", O_CREAT | O_RDWR | O_TEXT | O_TRUNC, 
  S_IREAD | S_IWRITE);
if (TmpFile == -1) 
    Err("Unable to create make2bat.tmp");
write (TmpFile,"", 1);
}

/***********************************************************************/
/*   FileWrite: Write a string to the temporary file                   */
/*                                                                     */
/*      INPUTS: StringPtr: Pointer to the string to be written         */
/*                                                                     */
/*     RETURNS: Offset of string in the file                           */
/*                                                                     */
/***********************************************************************/
long FileWrite(const char *StringPtr)
{
long RetCode;

RetCode = lseek(TmpFile, 0L, SEEK_END);
if (write(TmpFile, StringPtr, (unsigned int)(strlen(StringPtr) + 1)) == -1) 
    Err("Error writing to temporary file\n");
return RetCode;
}

/***********************************************************************/
/*    FileRead: Read a string from the temporary file                  */
/*                                                                     */
/*      INPUTS: Offset: Offset of the string to be read                */
/*                                                                     */
/*     RETURNS: Pointer to string                                      */
/*        NOTE: The pointer value returned points to a dynamically-    */
/*              allocated segment, and it is the caller's              */
/*              responsibility to return the segment.                  */
/***********************************************************************/
char *FileRead(long Offset)
{
char buff[LINELEN];

lseek(TmpFile, Offset, SEEK_SET);
if (read(TmpFile, buff, LINELEN) == -1)
    Err("Error reading from temporary file\n");
return MemNewString(buff);
}


