/* @(#) pSOSystem ARM/V2.2.0: bin/source/make2bat/err.c 2.3 96/12/20 17:49:59 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/make/err.c                                    */
/*   DATE:    96/12/20                                                 */
/*   PURPOSE: Error handling                                           */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1995, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
#include "global.h"
#include <stdio.h>
#include <stdlib.h>

/*---------------------------------------------------------------------*/
/* This is used for error handling with nested include files.  The     */
/* whole reason for this arrangement is so that if an error occurs     */
/* while processing an input file, the name of the file and the line   */
/* number can be reported.  NestLevel is an index pointing to the      */
/* element in the array corresponding to the current file.  If         */
/* NestLevel is 0, it indicates that there is not a file currently     */
/* being read.                                                         */
/*---------------------------------------------------------------------*/
#define MAX_NEST 16

static int NestLevel;
extern void clean_up(void);

static struct
    {
    char *filename;      /* pointer to name of file */
    int linenumber;      /* current line number in the file */
    } FileInfo[MAX_NEST + 1];


/***********************************************************************/
/*     ErrInit: Initialize this module                                 */
/*                                                                     */
/***********************************************************************/
void ErrInit(void)
{
NestLevel = 0;
}

/***********************************************************************/
/* ErrFilePush: Push a new file onto the nesting stack                 */
/*                                                                     */
/*      INPUTS: FileName: ptr to name of new file                      */
/*                                                                     */
/*        NOTE: This could actually generate an error of its own if    */
/*              the maximum nesting level were to be exceeded          */
/*                                                                     */
/***********************************************************************/
void ErrFilePush(const char *FileName)
{
if (NestLevel == (MAX_NEST + 1))
    ErrInt("Maximum include file nesting level of %d exceeded", MAX_NEST);

FileInfo[++NestLevel].filename = MemNewString(FileName);
FileInfo[NestLevel].linenumber = 0;
}

/***********************************************************************/
/* ErrFileIncLineno: Increment the current file's line number          */
/*                                                                     */
/***********************************************************************/
void ErrFileIncLineno(void)
{
FileInfo[NestLevel].linenumber++;
}

/***********************************************************************/
/*  ErrFilePop: Pop a file position entry off the nesting stack        */
/*                                                                     */
/***********************************************************************/
void ErrFilePop(void)
{
NestLevel--;
}

/***********************************************************************/
/*         Err: Report an error and terminate the program              */
/*                                                                     */
/*      INPUTS: msg: pointer to message string to print out            */
/*                                                                     */
/*     NOTE(S):                                                        */
/*                                                                     */
/***********************************************************************/
void Err(const char *msg)
{
if (NestLevel)
    fprintf(stderr, "%s line %d: %s\n", FileInfo[NestLevel].filename,
      FileInfo[NestLevel].linenumber, msg);
else
    fprintf(stderr, msg);

clean_up();
exit(-1);
}

/***********************************************************************/
/*      ErrInt: Generate a fatal error.  Error message may have an     */
/*              "int" to be expanded                                   */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/***********************************************************************/
void ErrInt(const char *msg, int arg)
{
char msgbuff[100];
sprintf(msgbuff, msg, arg);
Err(msgbuff);
}


/***********************************************************************/
/*      ErrPtr:                                                        */
/*                                                                     */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/***********************************************************************/
void ErrPtr(const char *msg, void *ptr)
{
char msgbuff[100];
sprintf(msgbuff, msg, ptr);
Err(msgbuff);
}

/***********************************************************************/
/*      ErrStr:                                                        */
/*                                                                     */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/***********************************************************************/
void ErrStr(const char *msg, const char *str)
{
char msgbuff[100];
sprintf(msgbuff, msg, str);
Err(msgbuff);
}

