/* @(#) pSOSystem ARM/V2.2.0: bin/source/make2bat/make.c 1.3 96/12/20 17:49:11 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/make/make.c                                   */
/*   DATE:    96/12/20                                                 */
/*   PURPOSE: Convert Makefiles to MS-DOS batch files                  */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1995, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
#include "global.h"
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#define LINELEN 2000
#define MAX_CMDLINE_MACROS 20

void ReadFile(char *FileName);
int SymbolLen(char *ptr);
int NonBlankLen(char *ptr);
static void ProcEquStrings(char **ArrayPtr, int where);
static int NextLine(char *BuffPtr, int len, FILE *infile);
void ParseRules(char *RulePtr, FILE *outfile);
void make2bat(int argc, char *argv[], char *BatchName);
static void TurnSlashes(char *pathptr);

#define SKIP_WHITE_SPACE(p) while ((*p == ' ') || (*p == '\t')) p++

static char BuildLabel[8], OkLabel[8];
static char *TargetName;


static void LabelIncrement(void);
int	E_opt_set = 0;
int s_opt = 0;      /* "silent" option flag */
static char m_opt;  /* "turn MS-DOS slashes" option flag */
char BatchFileName[14];

main(int argc, char *argv[])
{
struct stat sbuf;

/*---------------------------------------------------------------------*/
/* If there is a batch file left over, remove it.                      */
/*---------------------------------------------------------------------*/
strcpy(BatchFileName, "1.bat");
if (access(BatchFileName, 06) == 0) 
    {
    strcpy(BatchFileName, "2.bat");
    if (access(BatchFileName, 06) == 0) 
        {
        strcpy(BatchFileName, "3.bat");
        if (access(BatchFileName, 06) == 0) 
            {
            strcpy(BatchFileName, "4.bat");
            if (access(BatchFileName, 06) == 0) 
                {
                strcpy(BatchFileName, "5.bat");
                if (access(BatchFileName, 06) == 0) 
                    {
                    strcpy(BatchFileName, "6.bat");
                    if (access(BatchFileName, 06) == 0) 
                        {
                        strcpy(BatchFileName, "7.bat");
                        if (access(BatchFileName, 06) == 0) 
                            strcpy(BatchFileName, "8.bat");
                        }
                    }
                }
            }
        }
    }

/*---------------------------------------------------------------------*/
/* Produce a batch file and execute it.                                */
/*---------------------------------------------------------------------*/
make2bat(argc, argv, BatchFileName);
if (stat(BatchFileName, &sbuf))
    exit(1);
system(BatchFileName);
/*---------------------------------------------------------------------*/
/* Remove the batch file                                               */
/*---------------------------------------------------------------------*/
if (access(BatchFileName, 0) == 0) 
    remove(BatchFileName);
return 0;
}

FILE *outfile;

void clean_up(void)
{
/*---------------------------------------------------------------------*/
/* Remove the batch file                                               */
/*---------------------------------------------------------------------*/
if (access(BatchFileName, 0) == 0) 
    remove(BatchFileName);
}

/***********************************************************************/
/*    make2bat: Produce a batch file from the makefile                 */
/*                                                                     */
/*      INPUTS: argc: # of command line arguments                      */
/*              argv: ptr to array of argument pointers                */
/*              BatchName: Name of batch file to produce               */
/*                                                                     */
/***********************************************************************/
void make2bat(int argc, char *argv[], char *BatchName)
{
int i;
char *MakefileName;
char p_opt;
char *CmdLineMacros[MAX_CMDLINE_MACROS], **CmdLineMacroPtr;
int NrCmdLineMacros;

/*---------------------------------------------------------------------*/
/* Initialization                                                      */
/*---------------------------------------------------------------------*/
MemInit();
MacroInit();
ErrInit();
RulesInit();
strcpy(OkLabel, ":OK00");
strcpy(BuildLabel, ":BLD00");

MakefileName = (char *)0;
TargetName = (char *)0;
p_opt = 0;
m_opt = 0;

for (i = 0; i < MAX_CMDLINE_MACROS; i++) CmdLineMacros[i] = (char *)0;
CmdLineMacroPtr = CmdLineMacros;
NrCmdLineMacros = 0;

/*---------------------------------------------------------------------*/
/* Process command-line options                                        */
/*---------------------------------------------------------------------*/
for (i = 1; i < argc; i++)
    {
    if (*argv[i] == '-')
        {
        if (strcmp(&argv[i][1], "f") == 0)
            {
            /*---------------------------------------------------------*/
            /* -f option - next argument should be name of makefile    */
            /*---------------------------------------------------------*/
            if (argc == ++i) 
                Err("No argument given for -f option");
            MakefileName = argv[i];
            }
        else if (strcmp(&argv[i][1], "p") == 0)
            p_opt = 1;
        else if (strcmp(&argv[i][1], "e") == 0)
            E_opt_set = 1;
        else if (strcmp(&argv[i][1], "s") == 0)
            s_opt = 1;
        else if (strcmp(&argv[i][1], "m") == 0)
            m_opt = 1;
        else
            ErrStr("Unknown option: %s", argv[i]);
        }
    else if (strchr(argv[i], (int)'='))
        /*-------------------------------------------------------------*/
        /* Command-line macro definition.  Save a pointer to it.       */
        /* These are processed LAST.                                   */
        /*-------------------------------------------------------------*/
        {
        if (++NrCmdLineMacros > MAX_CMDLINE_MACROS)
            Err("Too many macros defined on command line");
        *CmdLineMacroPtr++ = argv[i];
        }
    else
        TargetName = argv[i];
    }


/*-------------------------------------------------------------------*/
/* These calls shall be made in the order M_CMD_LINE followed by     */
/* M_ENV. This way it makes the processing more efficient.           */
/*-------------------------------------------------------------------*/
ProcEquStrings(CmdLineMacros, M_CMD_LINE);
ProcEquStrings(environ, M_ENV);
    
if (MakefileName == (char *)0)
    MakefileName = "makefile";
ReadFile(MakefileName);

outfile = fopen(BatchName, "w");
if (outfile == NULL)
    ErrStr("Unable to open %s for output", BatchName);
fprintf(outfile, "echo off\n");

if (TargetName == (char *)0)
    Err("No target specified on command line or in makefile");
ParseRules(TargetName, outfile);

fprintf(outfile, " echo BUILD PROCESS COMPLETED SUCCESSFULLY\n");
fprintf(outfile, "goto :OK\n");
fprintf(outfile, ":ERROR\n");
fprintf(outfile, "echo BUILD PROCESS ABORTED DUE TO ERROR!\n");
fprintf(outfile, ":OK\n");

if (p_opt)
    {
    MacroDump();
    RulesDump();
    }
fclose(outfile);
}

/***********************************************************************/
/*    ReadFile: Read and process a "make" file                         */
/*                                                                     */
/*      INPUTS: FileName: pointer to name of file to process           */
/*                                                                     */
/***********************************************************************/
void ReadFile(char *FileName)
{
FILE *infile;
char LineBuffer[LINELEN];
char *LinePtr;
char *SymPtr, *MacValPtr, *TargetPtr, *DepPtr, *ptr;

/*-------------------------------------------------------------------*/
/* Open the file.  If we can't, we're off work early!                */
/*-------------------------------------------------------------------*/
infile = fopen(FileName, "r");
if (infile == NULL)
    ErrStr("Unable to open %s for input", FileName);
ErrFilePush(FileName);

/*-------------------------------------------------------------------*/
/* 
/*-------------------------------------------------------------------*/
while (NextLine(LineBuffer, LINELEN, infile))
    {
newline:
    LinePtr = LineBuffer;
    SKIP_WHITE_SPACE(LinePtr);

    /*---------------------------------------------------------------*/
    /* Strip off trailing comments                                   */
    /*---------------------------------------------------------------*/
    ptr = LinePtr;
    while (*ptr) 
        if (*ptr == '#') 
            {
            *ptr = 0;
            break;
            }
        else
            ptr++;

    /*---------------------------------------------------------------*/
    /* Comments and blank lines are easy.                            */
    /*---------------------------------------------------------------*/
    if (*LinePtr == 0) continue;
    if (*LinePtr == '#') continue;   /* Comment */

    /*---------------------------------------------------------------*/
    /* Put first symbol in *SymPtr, and advance LinePtr to next      */
    /* non-blank character                                           */
    /*---------------------------------------------------------------*/
    SymPtr = MacroExpand(LinePtr, SymbolLen(LinePtr));
    LinePtr = &LinePtr[SymbolLen(LinePtr)];
    SKIP_WHITE_SPACE(LinePtr);

    /*---------------------------------------------------------------*/
    /* If this is an include file, read it.                          */
    /*---------------------------------------------------------------*/
    if (strcmp(SymPtr, "include") == 0)
        {
        MemFree(SymPtr);
        while (*LinePtr == ' ') LinePtr++;
        SymPtr = MacroExpand(LinePtr, SymbolLen(LinePtr));
        ReadFile(SymPtr);
        MemFree(SymPtr);
        }
    
    /*---------------------------------------------------------------*/
    /* Check for a macro definition.  If it is a macro definition,   */
    /* the value of macro is everything from the first non-blank     */
    /* character following the '=' to the end of the line (except    */
    /* for any white space immediately preceeding the end of the     */
    /* line).                                                        */
    /*---------------------------------------------------------------*/
    else if (*LinePtr == '=')
        {
        LinePtr++;   /* Throw away the '=' */
        SKIP_WHITE_SPACE(LinePtr);
        ptr = &LinePtr[strlen(LinePtr) - 1];
        while ((*ptr == ' ') || (*ptr == '\t'))
            *ptr-- = 0;
        MacroDefine(SymPtr, LinePtr, M_FILE);
        MemFree(SymPtr);
        }

    /*---------------------------------------------------------------*/
    /* Check for rules definition.                                   */
    /*---------------------------------------------------------------*/
    else if (*LinePtr == ':')
        {
        LinePtr++;  /* Throw away the ':' */
        SKIP_WHITE_SPACE(LinePtr);
        TargetPtr = SymPtr;
        if (TargetName == (char *)0)
            TargetName = MemNewString(TargetPtr);
        RulesMake(TargetPtr);
        while (*LinePtr)
            {
            SKIP_WHITE_SPACE(LinePtr);
            SymPtr = MacroExpand(LinePtr, NonBlankLen(LinePtr));
            LinePtr = &LinePtr[NonBlankLen(LinePtr)];

            /*-------------------------------------------------------*/
            /* *SymPtr may actually be several dependencies separated */
            /* by white space, so add each one separately.           */
            /*-------------------------------------------------------*/
            ptr = SymPtr;
            while (*ptr)
                {
                SKIP_WHITE_SPACE(ptr);
                DepPtr = MemAlloc(NonBlankLen(ptr) + 1);
                strncpy(DepPtr, ptr, NonBlankLen(ptr));
                DepPtr[NonBlankLen(ptr)] = 0;
                RulesAddDependency(TargetPtr, DepPtr);
                MemFree(DepPtr);
                ptr = &ptr[NonBlankLen(ptr)];
                }
            MemFree(SymPtr);
            }
        while (NextLine(LineBuffer, LINELEN, infile))
            {
            if (LineBuffer[0] != '\t') 
                {
                MemFree(TargetPtr);
                goto newline;
                }
            else
                {
                LinePtr = LineBuffer;
                SKIP_WHITE_SPACE(LinePtr);
                TurnSlashes(LinePtr);
                RulesAddAction(TargetPtr, LinePtr);
                }
            }
        }

    /*---------------------------------------------------------------*/
    /* Don't know WHAT it is!                                        */
    /*---------------------------------------------------------------*/
    else
        {
        MemFree(SymPtr);
        Err("Syntax error");
        }
    }

fclose(infile);
ErrFilePop();
}

/***********************************************************************/
/*   SymbolLen: Determine the length of a symbol                       */
/*                                                                     */
/*      INPUTS: ptr: Pointer to symbol                                 */
/*                                                                     */
/*     RETURNS: Length of symbol                                       */
/*       NOTES: Any of the following are considered symbol delimiters: */
/*              NULL, newline, ":", space, "=", and tab                */
/*                                                                     */
/***********************************************************************/
int SymbolLen(char *ptr)
{
int i;

i = 0;
while ((*ptr != 0) && (*ptr != '\n') && (*ptr != ':') && (*ptr != ' ')
  && (*ptr != '=') && (*ptr != '\t'))
    {
    ptr++;
    i++;
    }
return i;
}

/***********************************************************************/
/* NonBlankLen: Return length of a "non-blank" string                  */
/*                                                                     */
/*      INPUTS: ptr: points to string to take length of                */
/*                                                                     */
/*     RETURNS: Length of string up to but not including,  first NULL, */
/*              tab, or space                                          */
/*                                                                     */
/***********************************************************************/
int NonBlankLen(char *ptr)
{
int i;

i = 0;
while ((*ptr != 0) && (*ptr != ' ') && (*ptr != '\t'))
    {
    ptr++;
    i++;
    }
return i;
}

/***********************************************************************/
/*    NextLine:                                                        */
/*                                                                     */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/*     RETURNS:                                                        */
/*     OUTPUTS:                                                        */
/*     NOTE(S):                                                        */
/*                                                                     */
/***********************************************************************/
static int NextLine(char *BuffPtr, int len, FILE *infile)
{
char *lp;

if (fgets(BuffPtr, len, infile) != NULL)
    {
    ErrFileIncLineno();

    /*---------------------------------------------------------------*/
    /* Get rid of trailing line feed, if present.                    */
    /*---------------------------------------------------------------*/
    if (BuffPtr[strlen(BuffPtr) - 1] == '\n') 
        BuffPtr[strlen(BuffPtr) - 1] = 0;

    lp = BuffPtr;
    /*---------------------------------------------------------------*/
    /* Handle lines which are continued.                             */
    /*---------------------------------------------------------------*/
    while (lp[strlen(lp) - 1] == '\\')
        {
        lp[strlen(lp) - 1] = 0;  /* Erase the '\' */
        lp = &lp[strlen(lp)];    /* Advance ptr to first byte past end */
        if (strlen(BuffPtr) > (len - 80))
            Err("Line too long");
        if (fgets(lp, (len - strlen(BuffPtr)), infile) == NULL)
            Err("Unexpected end-of-file");
        ErrFileIncLineno();
        if (BuffPtr[strlen(BuffPtr) - 1] == '\n') 
            BuffPtr[strlen(BuffPtr) - 1] = 0;
        }

    return 1;
    }

return 0;
}


/***********************************************************************/
/*  ParseRules:                                                        */
/*                                                                     */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/*     RETURNS:                                                        */
/*     OUTPUTS:                                                        */
/*     NOTE(S):                                                        */
/*                                                                     */
/***********************************************************************/
void ParseRules(char *target, FILE *outfile)
{
int i;
int found_squote=0;
int found_dquote=0;
char *DepPtr, *ActPtr, *ExpandedActPtr, *ptr, *p;

StartOver:
for (i = 1; ; i++)
    {
    DepPtr = RulesFindDependency(target, i);
    if (DepPtr == (char *)0) break;

    if (RulesFind(DepPtr, 1)) 
        {
        ParseRules(DepPtr, outfile);
        MemFree(DepPtr);
        goto StartOver;
        }
    else
        MemFree(DepPtr);
    }

/*---------------------------------------------------------------------*/
/* If the target has any dependencies, emit batch file code to check   */
/* them and skip the actions if everything is up-to-date.  Note that   */
/* if a target has NO dependencies, the actions are always taken.      */
/*---------------------------------------------------------------------*/
if (RulesFindDependency(target, 1) != (char *)0)
    {
    for (i = 1; ; i++)
        {
        DepPtr = RulesFindDependency(target, i);
        if (DepPtr == (char *)0) break;
    
        fprintf(outfile, "ckdepend %s %s\n", target, DepPtr);
        fprintf(outfile, "if errorlevel 1 goto %s\n", BuildLabel);
        }
    fprintf(outfile, "goto %s\n", OkLabel);
    fprintf(outfile, "%s\n", BuildLabel);
    }

/*---------------------------------------------------------------------*/
/* Run through the actions and do them.                                */
/* NEED TO ADD CODE TO HANDLE MACRO DEFINITIONS!
/*---------------------------------------------------------------------*/
for (i = 1; ; i++)
    {
    if ((ActPtr = RulesFindAction(target, i)) == (char *)0) break;
    ExpandedActPtr = MacroExpand(ActPtr, strlen(ActPtr));
    TurnSlashes(ExpandedActPtr);
    if ((*ExpandedActPtr != '@') && (!s_opt))
        fprintf(outfile, "echo \"%s\"\n", ExpandedActPtr);

    for (p = ExpandedActPtr; *p; p++) 
        {
        if (!found_squote && *p == '\"')
            {
	    if(!found_dquote)
	        {
	        *p = ' ';
	        found_dquote = 1;
	        continue;
                }
            else		 
    	        {
    	        found_dquote = 0;
	        *p = ' ';
	        continue;
	        }
            }

	if (!found_dquote && *p == '\'')
	    {
	    if(!found_squote)
	        {
	        *p = ' ';
	        found_squote = 1;
	        continue;
	        }
	    else
    	        {
    	        found_squote = 0;
	        *p = ' ';
	        continue;
	        }
	    }
	    
        if (!found_dquote && !found_squote && *p == ';')
            *p = '\n';
        }

    if (*(p-1) == '/')
        *(p-1) = ' ';

    fprintf(outfile, "%s\n", ExpandedActPtr);
    if ((strncmp(ExpandedActPtr, "echo ", 5) != 0) 
     && (strncmp(ExpandedActPtr, "@echo ", 6) != 0))
        fprintf(outfile, "if errorlevel 1 goto error\n");
    MemFree(ActPtr);
    MemFree(ExpandedActPtr);
    }
fprintf(outfile, "%s\n", OkLabel);
LabelIncrement();

RulesMarkBuilt(target);
}



/***********************************************************************/
/* LabelIncrement:                                                     */
/*                                                                     */
/*      INPUTS: var1:                                                  */
/*                                                                     */
/*     RETURNS:                                                        */
/*     OUTPUTS:                                                        */
/*     NOTE(S):                                                        */
/*                                                                     */
/***********************************************************************/
static void LabelIncrement(void)
{
char *ptr;
int i;

for (ptr = &OkLabel[3], i = 0; i < 2; ptr = &BuildLabel[4], i++)
    {
    if (ptr[1] == '9')
        {
        ptr[0]++;
        ptr[1] = '0';
        }
    else
        ptr[1]++;
    }
}

/***********************************************************************/
/* ProcEquStrings: Process an array of strings of form "A=B" as macro  */
/*              definitions                                            */
/*      INPUTS: ArrayPtr: pointer to array of character string pointers*/
/*                                                                     */
/***********************************************************************/
static void ProcEquStrings(char **ArrayPtr, int where)
{
char *EnvStringPtr, *ValueStringPtr;
char NullString[2];

NullString[0] = 0;
while (*ArrayPtr)
    {
    EnvStringPtr = MemNewString(*ArrayPtr++);
    if (*EnvStringPtr)   /* watch out for empty strings! */
        {
        ValueStringPtr = strchr(EnvStringPtr, (int)'=');
        if (ValueStringPtr == NULL)
            ValueStringPtr = NullString;
        else
            *ValueStringPtr++ = 0; 
        while (*ValueStringPtr == ' ') ValueStringPtr++;  /* skip spaces */
        MacroDefine(EnvStringPtr, ValueStringPtr, where);
        MemFree(EnvStringPtr);
        }
    }
}

/***********************************************************************/
/* TurnSlashes: Convert a pathname to MS-DOS style                     */
/*                                                                     */
/*      INPUTS: pathptr: ptr to pathname to change                     */
/*       NOTES: Does nothing unless -m option was specified!           */
/*                                                                     */
/***********************************************************************/
static void TurnSlashes(char *pathptr)
{
if (!m_opt) return;
while (*pathptr)
    {
    if (*pathptr == '/') *pathptr = '\\';
    pathptr++;
    }
}

