/* @(#) pSOSystem ARM/V2.2.0: bin/source/make2bat/mem.c 2.5 95/12/13 19:32:47 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/make/mem.c                                    */
/*   DATE:    95/12/13                                                 */
/*   PURPOSE: Dynamic memory allocation                                */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1995, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
#include "global.h"
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

/***********************************************************************/
/*     MemInit: Initialize the memory module                           */
/*                                                                     */
/*        NOTE: This must be called before any other routine in this   */
/*              module can be called.                                  */
/***********************************************************************/
void MemInit(void)
{
}

/***********************************************************************/
/*    MemAlloc: Allocate a dynamic memory segment                      */
/*                                                                     */
/*      INPUTS: nbytes: Number of bytes required                       */
/*                                                                     */
/*     RETURNS: Pointer to segment                                     */
/*        NOTE: This always returns a segment.  If a segment cannot be */
/*              allocated, a fatal error is generated.                 */
/*                                                                     */
/***********************************************************************/
char *MemAlloc(int nbytes)
{
char *ptr;

if (nbytes == 0) return (char *)0;

ptr = (char *)malloc(nbytes);
if (ptr == (char *)0) 
    ErrInt("MemAlloc: No more segments of size %d bytes available", nbytes);
return ptr;
}

/***********************************************************************/
/* MemNewString: Allocate a new segment and copy a string to it.       */
/*                                                                     */
/*      INPUTS: string: pointer to string to be cloned                 */
/*                                                                     */
/*     RETURNS: pointer to new copy of string                          */
/*                                                                     */
/***********************************************************************/
char *MemNewString(const char *string)
{
char *newptr;

newptr = MemAlloc(strlen(string) + 1); 
strcpy(newptr, string);
return newptr;
}

/***********************************************************************/
/*     MemFree: De-allocate a dynamic memory segment                   */
/*                                                                     */
/*      INPUTS: seg: pointer to segment being returned                 */
/*                                                                     */
/***********************************************************************/
void MemFree(char *seg)
{
if (seg) free(seg);
}

