/* @(#) pSOSystem ARM/V2.2.1: bin/source/mklibmk.c  1.2 97/05/14 15:51:53 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/mklibmk.c                                     */
/*   DATE:    97/05/14                                                   */
/*   PURPOSE: Generates OBJ = component.o                              */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1993, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/* This utility finds accepts two arguments. The first argument is the */
/* name of the directory to search and the second argument is an ascii */
/* file containing names of the files to be searched. A sample of the  */
/* file is given below. The string // is a comment. any text after this*/
/* ignored. Blank lines are also ignored.                              */
/*                                                                     */
/*                               //just a comment line                 */
/* file1 // file to be searched                                        */
/* file2                                                               */
/* file3                 // another file                               */
/*                                                                     */
/* The output from this program is of the form:                        */
/*                                                                     */
/* OBJS =  file1 file2 file3                                           */
/* COBJS=  file1,file2,file3                                           */
/*                                                                     */
/* These are intended for use in Makefiles                             */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

/*---------------------------------------------------------------------*/
/* Symbol Definitions                                                  */
/*---------------------------------------------------------------------*/
#define FLNAME_SZ   100          /* size of filename buffer */
#define OUTNAME_SZ  512          /* size of Output buffers */
#define LN_SIZE     512          /* size of input buffer */
#define DIR_CHAR    "/"          /* single slash */
#define SLASH       '/'          /* single slash */
#define SLASHS      '//'         /* comments in input file */
#define OUT_NM_PRE1 "OBJS"       /* beginning of first output line */
#define OUT_NM_PRE2 "OBJS1"      /* beginning of second output line */
#define OUT_NM_PRE3 "OBJS2"      /* beginning of third output line */
#define OUT_NM_PRE4 "OBJS3"      /* beginning of fourth output line */
#define OUT_NM_PRE5 "OBJS4"      /* beginning of fifth output line */
#define OUT_NM_PRE6 "OBJS5"      /* beginning of sixth output line */
#define OUT_NM_PRE7 "OBJS6"      /* beginning of seventh output line */
#define ASSIGN      " = "

static char *getFlNm();

char  tmpBuff[FLNAME_SZ];
char  flNmBuff[FLNAME_SZ];
char  foNmBuff[FLNAME_SZ];
char  outNmBuff1[OUTNAME_SZ];
char  outNmBuff2[OUTNAME_SZ];
char  outNmBuff3[OUTNAME_SZ];
char  outNmBuff4[OUTNAME_SZ];
char  outNmBuff5[OUTNAME_SZ];
char  outNmBuff6[OUTNAME_SZ];
char  outNmBuff7[OUTNAME_SZ];
char  lnBuff[LN_SIZE];

/***********************************************************************/
/* main:   This routine generates a string defining a variable         */
/*         to be used by makefiles for compiling components            */
/*                                                                     */
/* INPUTS: "Directory" and filename                                    */
/*                                                                     */
/* OUTPUTS: string "OBJS  = ksarml.o ..."                              */
/*                                                                     */
/***********************************************************************/
main (argc, argv)
int argc;
char *argv[];
{
FILE  *fd, *pd;

char *dirNmPtr, *wp;
char *flNmPtr,  *nmPtr, *extPtr, *tmpPtr;

int i, tmp, j;

dirNmPtr = *(++argv);
flNmPtr  = *(++argv);
extPtr  = *(++argv);

/*---------------------------------------------------------------------*/
/*  Check if the correct number of arguments are supplied.             */
/*---------------------------------------------------------------------*/
if (argc != 4)
   {
   printf("Usage: mklibmk dirname filename type[32l | 32b | 16l | 16b]\n");
   exit (0);
   }

wp = dirNmPtr;

strcpy(flNmBuff, wp);
strcpy(tmpBuff, flNmBuff);

/*---------------------------------------------------------------------*/
/*  Open the file name specified in the second argument.               */
/*---------------------------------------------------------------------*/
if ((fd = fopen (flNmPtr, "r")) == (FILE *) NULL)
   {
   printf("Unable to open file %s\n", flNmPtr);
   exit(-1);
   }

/*---------------------------------------------------------------------*/
/*  Copy the initial portion of the output string in the output buffer */
/*---------------------------------------------------------------------*/
strcpy(outNmBuff1, OUT_NM_PRE1);
strcpy(outNmBuff2, OUT_NM_PRE2);
strcpy(outNmBuff3, OUT_NM_PRE3);
strcpy(outNmBuff4, OUT_NM_PRE4);
strcpy(outNmBuff5, OUT_NM_PRE5);
strcpy(outNmBuff6, OUT_NM_PRE6);
strcpy(outNmBuff7, OUT_NM_PRE7);
strcat(outNmBuff1, ASSIGN);
strcat(outNmBuff2, ASSIGN);
strcat(outNmBuff3, ASSIGN);
strcat(outNmBuff4, ASSIGN);
strcat(outNmBuff5, ASSIGN);
strcat(outNmBuff6, ASSIGN);
strcat(outNmBuff7, ASSIGN);
strcat(outNmBuff1, "$(");
strcat(outNmBuff1, OUT_NM_PRE2);
strcat(outNmBuff1, ") $(");
strcat(outNmBuff1, OUT_NM_PRE3);
strcat(outNmBuff1, ") $(");
strcat(outNmBuff1, OUT_NM_PRE4);
strcat(outNmBuff1, ") $(");
strcat(outNmBuff1, OUT_NM_PRE5);
strcat(outNmBuff1, ") $(");
strcat(outNmBuff1, OUT_NM_PRE6);
strcat(outNmBuff1, ") $(");
strcat(outNmBuff1, OUT_NM_PRE7);
strcat(outNmBuff1, ")");

/*---------------------------------------------------------------------*/
/*  Read the file a line at a time, and extract the file names from    */
/*  the line just read. Then prepend the directory name to the file    */
/*  name. Attempt to open the file. If it opens then the file is       */
/*  present.                                                           */
/*---------------------------------------------------------------------*/
i = 0;
while ((wp = fgets (lnBuff, LN_SIZE, fd)) != (char *) NULL)
      {
      if ((nmPtr = getFlNm(wp)) != (char *) NULL)
         {
         int isobj = 0;

         strcpy(flNmBuff, tmpBuff);
         strcat(flNmBuff, nmPtr);
         if ((tmpPtr = strrchr(nmPtr, '.')) == NULL)
             {
             strcat(flNmBuff, ".s");
             ++isobj;
             }
         else if ((j = strcmp(tmpPtr+1,extPtr)) != 0)
	     continue;

         if ((pd = fopen (flNmBuff, "r")) != (FILE *) NULL)
            {
            fclose(pd);
            
            tmp = i%6;

            if(tmp == 0)
                {
		if (isobj)
		    {
                    strcat(outNmBuff2, "obj");
                    strcat(outNmBuff2, extPtr);
                    strcat(outNmBuff2, "/");
		    }
                strcat(outNmBuff2, nmPtr);
                if (isobj) strcat(outNmBuff2, ".o");
                strcat(outNmBuff2, " ");
                }
            else if (tmp == 1)
                {
		if (isobj)
		    {
                    strcat(outNmBuff3, "obj");
                    strcat(outNmBuff3, extPtr);
                    strcat(outNmBuff3, "/");
		    }
                strcat(outNmBuff3, nmPtr);
                if (isobj) strcat(outNmBuff3, ".o");
                strcat(outNmBuff3, " ");
                }
            else if (tmp == 2)
                {
		if (isobj)
		    {
                    strcat(outNmBuff4, "obj");
                    strcat(outNmBuff4, extPtr);
                    strcat(outNmBuff4, "/");
		    }
                strcat(outNmBuff4, nmPtr);
                if (isobj) strcat(outNmBuff4, ".o");
                strcat(outNmBuff4, " ");
                }
            else if (tmp == 3)
                {
		if (isobj)
		    {
                    strcat(outNmBuff5, "obj");
                    strcat(outNmBuff5, extPtr);
                    strcat(outNmBuff5, "/");
		    }
                strcat(outNmBuff5, nmPtr);
                if (isobj) strcat(outNmBuff5, ".o");
                strcat(outNmBuff5, " ");
                }
            else if (tmp == 4)
                {
		if (isobj)
		    {
                    strcat(outNmBuff6, "obj");
                    strcat(outNmBuff6, extPtr);
                    strcat(outNmBuff6, "/");
		    }
                strcat(outNmBuff6, nmPtr);
                if (isobj) strcat(outNmBuff6, ".o");
                strcat(outNmBuff6, " ");
                }
            else if (tmp == 5)
                {
		if (isobj)
		    {
                    strcat(outNmBuff7, "obj");
                    strcat(outNmBuff7, extPtr);
                    strcat(outNmBuff7, "/");
		    }
                strcat(outNmBuff7, nmPtr);
                if (isobj) strcat(outNmBuff7, ".o");
                strcat(outNmBuff7, " ");
                }
            i++;
            }
         }
      }

outNmBuff2[strlen(outNmBuff2) - 1] = 0;  /* Stomp out trailing " " */
outNmBuff3[strlen(outNmBuff3) - 1] = 0;  /* Stomp out trailing " " */
outNmBuff4[strlen(outNmBuff4) - 1] = 0;  /* Stomp out trailing " " */
outNmBuff5[strlen(outNmBuff5) - 1] = 0;  /* Stomp out trailing " " */
outNmBuff6[strlen(outNmBuff6) - 1] = 0;  /* Stomp out trailing " " */
outNmBuff7[strlen(outNmBuff7) - 1] = 0;  /* Stomp out trailing " " */

if ((j = strlen(outNmBuff3)) == 7)
    strcat(&outNmBuff3[7], &outNmBuff2[7]);

if ((j = strlen(outNmBuff4)) == 7)
    strcat(&outNmBuff4[7], &outNmBuff2[7]);

if ((j = strlen(outNmBuff5)) == 7)
    strcat(&outNmBuff5[7], &outNmBuff2[7]);

if ((j = strlen(outNmBuff6)) == 7)
    strcat(&outNmBuff6[7], &outNmBuff2[7]);

if ((j = strlen(outNmBuff7)) == 7)
    strcat(&outNmBuff7[7], &outNmBuff2[7]);

printf("%s \n", outNmBuff2);
printf("%s \n", outNmBuff3);
printf("%s \n", outNmBuff4);
printf("%s \n", outNmBuff5);
printf("%s \n", outNmBuff6);
printf("%s \n", outNmBuff7);
printf("%s \n", outNmBuff1);

fclose (fd);

return 0;
}

/***********************************************************************/
/* getFlnm: This routine extracts the file name from the input ptr to  */
/*          It discards blank lines and comments from the input string */
/*                                                                     */
/* INPUTS:  Pointer to the line to be processed.                       */
/*                                                                     */
/* OUTPUTS: Pointer to the name string if one exeists otherwise it     */
/*          returns NULL.                                              */
/*                                                                     */
/***********************************************************************/

static char *getFlNm (lnPtr)
char *lnPtr;
{
char *tmpPtr;

tmpPtr = lnPtr;

while ((*tmpPtr == ' ') || (*tmpPtr == '\t') || (*tmpPtr == '\n'))
      tmpPtr++;

if ((*tmpPtr == (char) NULL) || ((*tmpPtr == '/') && (*(tmpPtr + 1) == '/')))
   return ((char *) NULL);

while ((*tmpPtr != ' ') && (*tmpPtr != '\t') && (*tmpPtr != '\n'))
      {
      if ((*tmpPtr == '/') && (*(tmpPtr + 1) == '/'))
         break;
      else
         tmpPtr++;
      }

*tmpPtr = '\0';

return (lnPtr);
}

/******************************************************************************/

