/* @(#) pSOSystem ARM/V2.2.1: bin/source/rm.c  1.2 97/05/14 15:51:54 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bin/source/rm.c                                          */
/*   DATE:    97/05/14                                                 */
/*   PURPOSE: "rm" utility for MS-DOS                                  */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1993, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*   This is a UNIX-style "rm" utility for use on MS-DOS systems.      */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <dos.h>
#include <direct.h>
#include <ctype.h>
#include <fcntl.h>
#include <conio.h>
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>

extern int errno;

#define MAXPATHLEN  400

/*----------------------------------------------------------------------*/
/* Function prototypes                                                  */
/*----------------------------------------------------------------------*/
static void abort(char *msg);
static void DelDir(char *dst_path);
static void TurnSlashes(char *p);
static void DelPat(char *PathIn, unsigned char Ropt, unsigned char Fopt);

/************************************************************************/
/*        main: Main body of the program                                */
/*                                                                      */
/************************************************************************/
main(int argc, char *argv[])
{
unsigned char Ropt, Fopt;
char CwdPathname[MAXPATHLEN], path_string[MAXPATHLEN];
int CurrentArg;

/*---------------------------------------------------------------------*/
/* Check for options.  Options must appear BEFORE any filenames.  The  */
/* only options this utility recognizes are -r and -f.                 */
/*---------------------------------------------------------------------*/
Ropt = 0;
Fopt = 0;
CurrentArg = 1;

while (argv[CurrentArg][0] == '-')
    {
    switch (argv[CurrentArg][1])
        {
        case 'f': Fopt = 1; break;
        case 'r': Ropt = 1; break;
        default: abort("Invalid option");
        }
    CurrentArg++;
    }

/*---------------------------------------------------------------------*/
/* Save the pathname of the current directory                          */
/*---------------------------------------------------------------------*/
getcwd(CwdPathname, MAXPATHLEN);

/*---------------------------------------------------------------------*/
/* Remaining arguments are assumed to be filenames to delete           */
/*---------------------------------------------------------------------*/
while (argv[CurrentArg])
    {
    TurnSlashes(argv[CurrentArg]);
    if (strchr(argv[CurrentArg], ':'))
        {
        if (!Fopt)
            printf("rm using volume specification not supported.\n");
        }
    else
        {
        if (argv[CurrentArg][0] == '\\')
            {
            strcpy(path_string, argv[CurrentArg]);
            }
        else
            {
            strcpy(path_string, CwdPathname);
            strcat(path_string, "\\");
            strcat(path_string, argv[CurrentArg]);
            }

        DelPat(path_string, Ropt, Fopt);
        }

    chdir(CwdPathname);
    CurrentArg++;
    }

exit(0); 
}


/***********************************************************************/
/*      DelPat: Delete the files/directories that match a pattern      */
/*                                                                     */
/*      INPUTS: PathIn = the path with the pattern in it               */
/*              Ropt   = recusive option for directories               */
/*              Fopt   = force option (silent mode)                    */
/*                                                                     */
/***********************************************************************/
static void DelPat(char *PathIn, unsigned char Ropt, unsigned char Fopt)
{
struct find_t findbuff;
unsigned int rc;
int pos;
char pathname[MAXPATHLEN], PathBuff[MAXPATHLEN];
char *Pattern, *Path;
 
strcpy(PathBuff, PathIn);
for (pos = strlen(PathBuff) - 1; (pos >= 0); pos--)
    if (PathBuff[pos] == '\\')
        {
       if (pos == 0)
            {
            PathBuff[++pos] = 0;   /* Terminate path */
            break;
            }
        else if (pos != strlen(PathBuff) - 1)
            {
            PathBuff[pos++] = 0;    /* Terminate path */
            break;
            }
        else
            {
            PathBuff[pos] = 0;     /* null out trailing \ */
            PathIn[pos] = 0;       /* null out trailing \ */
            }
        }
if (pos > 0)
    {
    /*-----------------------------------------------------------------*/
    /* If there is a path, chdir to that path.                         */
    /*-----------------------------------------------------------------*/
    rc = chdir(PathBuff);
    if (rc)
        {
        if (!Fopt)
            printf("%s does not exist.\n", PathIn);
        return;
        }
    /*-----------------------------------------------------------------*/
    /* Copy the pattern part to replace the path.                      */
    /*-----------------------------------------------------------------*/
    strcpy(PathBuff, &PathIn[pos]);
    }

rc = _dos_findfirst(PathBuff, _A_NORMAL | _A_RDONLY | _A_SUBDIR |
                    _A_ARCH | _A_HIDDEN , &findbuff);
if (rc)
    {
    if (!Fopt)
        printf("%s does not exist.\n", PathIn);
    return;
    }

while (rc == 0)
    {
    if ((strcmp(findbuff.name, ".") != 0) &&
            (strcmp(findbuff.name, "..") != 0))
        {
        getcwd(pathname, MAXPATHLEN);
        if (pathname[strlen(pathname) - 1] != '\\')
            strcat(pathname, "\\");
        strcat(pathname, findbuff.name);

        if (findbuff.attrib & _A_SUBDIR)
            {
            if (!Ropt) 
                {
                if (!Fopt)
                    printf("%s is a directory. Not removed.\n",
                           pathname);
                }
            else
                DelDir(pathname);
            }
        else if (findbuff.attrib & _A_RDONLY)
            {
            if (!Fopt)
                printf("%s is read only. Not removed.\n",
                       pathname);
            else
                {
                chmod(pathname, S_IWRITE | S_IREAD);
                remove(pathname);
                }
            }
        else
            remove(pathname);
        }
    rc = _dos_findnext(&findbuff);
    }
}


/************************************************************************/
/*     DelDir: Recursively delete a directory                           */
/*                                                                      */
/*      INPUTS: DirPath: pointer to source pathname                     */
/*                                                                      */
/************************************************************************/
static void DelDir(DirPath)
char *DirPath;
{
struct find_t findbuff;
unsigned int rc;
char pathname[MAXPATHLEN];

chdir(DirPath);
rc = _dos_findfirst("*.*", _A_NORMAL | _A_HIDDEN | _A_SUBDIR, &findbuff);
while (rc == 0)
    {
    if ((strcmp(findbuff.name, ".") != 0) &&
            (strcmp(findbuff.name, "..") != 0))
        {
        strcpy(pathname, DirPath);
        strcat(pathname, "\\");
        strcat(pathname, findbuff.name);

        if (findbuff.attrib & _A_SUBDIR)
            DelDir(pathname);
        else
            remove(pathname);
        }
    rc = _dos_findnext(&findbuff);
    }
chdir("..");
rmdir(DirPath);
}


/***********************************************************************/
/*       abort: Terminate the program with an error message            */
/*                                                                     */
/*      INPUTS: msg: pointer to ASCIIZ error message                   */
/*                                                                     */
/***********************************************************************/
static void abort(msg)
char *msg;
{
printf("rm: %s\n", msg);
exit(-1);
}


/***********************************************************************/
/* TurnSlashes: Convert a UNIX-style pathname to MS-DOS compatible     */
/*                                                                     */
/*      INPUTS: p: Pointer to pathname to convert                      */
/*                                                                     */
/*     OUTPUTS: *p has "/"s changed to "\"s.                           */
/*                                                                     */
/***********************************************************************/
static void TurnSlashes(char *p)
{
while (*p)
    {
    if (*p == '/') *p = '\\';
    p++;
    }
}
